from spacy.symbols import (
    ADJ,
    ADP,
    ADV,
    AUX,
    CCONJ,
    DET,
    INTJ,
    NOUN,
    NUM,
    PART,
    PRON,
    PROPN,
    PUNCT,
    SCONJ,
    SYM,
    VERB,
    X,
)

TAG_MAP = {
    "NOUN": {"pos": NOUN},
    "VERB": {"pos": VERB},
    "AUX": {"pos": AUX},
    "ADJ": {"pos": ADJ},
    "ADV": {"pos": ADV},
    "PRON": {"pos": PRON},
    "DET": {"pos": DET},
    "ADP": {"pos": ADP},
    "SCONJ": {"pos": SCONJ},
    "CCONJ": {"pos": CCONJ},
    "PART": {"pos": PART},
    "INTJ": {"pos": INTJ},
    "NUM": {"pos": NUM},
    "PROPN": {"pos": PROPN},
    "PUNCT": {"pos": PUNCT},
    "SYM": {"pos": SYM},
    "X": {"pos": X},
}
